@@ -16,15 +16,14 @@ from . import exceptions
 
 class TorchTagger:
 
-    def __init__(self, embedding, model_arch="fastText", n_classes=2, num_epochs=5):
+    def __init__(self, embedding, model_arch="fastText", tokenizer=None):
         # retrieve model and initial config
         self.description = None
         self.config = TORCH_MODELS[model_arch]["config"]()
         self.model_arch = TORCH_MODELS[model_arch]["model"]
-        # set number of output classes
-        self.config.output_size = n_classes
-        # set number of epochs
-        self.config.max_epochs = num_epochs
+        # output size & max epochs
+        self.config.output_size = None
+        self.config.max_epochs = None
         # statistics report for each epoch
         self.epoch_reports = []
         # model
@@ -37,13 +36,15 @@ class TorchTagger:
         self.tokenizer = self._get_tokenizer()
         self.embedding = embedding
         # text processor
-        self.text_processor = TextProcessor(phraser=embedding.phraser, remove_stop_words=False)
+        self.text_processor = TextProcessor(phraser=embedding.phraser, remove_stop_words=False, tokenizer=tokenizer)
 
 
     @staticmethod
     def _get_tokenizer():
-        # TODO: replace with normal tokenizer
-        # my first hacky tokenizer
+        """
+        Retrieves simple tokenizer. As we are using MLP to tokenize texts, let's split only on whitespace.
+        :return: Tokenizer as a lambda function.
+        """
         return lambda sent: [x.strip() for x in sent.split(" ")]
 
 
@@ -59,6 +60,11 @@ class TorchTagger:
 
     @staticmethod
     def evaluate_model(model, iterator):
+        """
+        Evaluates the model using evaluation set & Tagging Report.
+        :param: iterator: Instance of TorchText BucketIterator.
+        :return: Instance of Tagging Report containing the results.
+        """
         all_preds = []
         all_y = []
         for idx, batch in enumerate(iterator):
@@ -77,17 +83,20 @@ class TorchTagger:
         return report
 
 
-    def train(self, data_sample):
+    def train(self, data_sample, num_epochs=5):
         """
         Trains model based on data sample.
         :param: dict data_sample: Dictonary containing class labels as keys and lists of examples as values.
         :return: TaggingReport object.
         """
+        # set max epochs
+        self.config.max_epochs = num_epochs
+        # clear old epoch reports
+        self.epoch_reports = []
         # prepare data
         train_iterator, val_iterator, test_iterator, text_field = self._prepare_data(data_sample)
         # declare model
         model = self.model_arch(self.config, len(text_field.vocab), text_field.vocab.vectors, self.evaluate_model)
-
         # check cuda
         if torch.cuda.is_available():
             model.cuda()
@@ -103,7 +112,6 @@ class TorchTagger:
         for i in range(self.config.max_epochs):
             report = model.run_epoch(train_iterator, val_iterator, i)
             self.epoch_reports.append(report)
-            #print("Epoch:", i, report.to_dict())
         # set model
         self.model = model
         # set vocab
@@ -165,8 +173,7 @@ class TorchTagger:
         # retrieve examples for each class
         for label, class_examples in data_sample.items():
             for example in class_examples:
-                # TODO: will this throw indexerror?
-                example = self.text_processor.process(example)[0]
+                example = self.text_processor.process(example)
                 examples.append(example)
                 labels.append(self.label_index[label])
         return examples, labels
@@ -239,8 +246,7 @@ class TorchTagger:
         :return: class number, class probability
         """
         # process text with our text processor
-        # TODO: will this throw indexerror?
-        text = self.text_processor.process(text)[0]
+        text = self.text_processor.process(text)
         # process text with torchtext processor
         processed_text = self.text_field.process([self.text_field.preprocess(text)])
         # check cuda
@@ -276,6 +282,6 @@ class TorchTagger:
     @staticmethod
     def _process_prediction_output(predicted_label, probability):
         return {
-            "prediction": bool(predicted_label),
+            "prediction": predicted_label,
             "probability": probability
         }
